/*
  LCD - Library for most LCD charater displays, has common features 
  Copyright (c) 2013 Jim Spence.  All right reserved.
  www.byvac.com - see terms and conditions for using hardware
  
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  
  Release Notes:
  p017 is an LCD chip that can be supplied with or without a PCB.
  p017 - serial varsion
  p018 - i2c version

*/  

#include "Arduino.h"
#include "p017.h"

// =============================================================================
// SV3B Type serial control
// =============================================================================
// =============================================================================
// *****************************************************************************
// constructor is just a connection to NewSoftSerial
// *****************************************************************************
LCD::LCD(uint8_t rxPin, uint8_t txPin, long rbaud, char address) : BSerial(rxPin,txPin)
{
    _ack = 6;
    _adr = address;
    baud(rbaud);
}

// *****************************************************************************
// Clear screen on most displays is cmd 1
// *****************************************************************************
void LCD::lcd_cls()
{
    putch(_adr);
    puts("c1\r");
    delay(300);
}

// *****************************************************************************
// select a row and column
// row and column start at 0
// *****************************************************************************
void LCD::lcd_rowcol(char row, char col)
{
const int lcdrows[]={0x80,0xc0,0x94,0xd4}; 
int rc;
char buf[15];
    if(row < 0) row = 0;
    if(row > 3) row = 3;
    rc = lcdrows[row] + col;
    putch(_adr);
    sprintf(buf, "c%d\r", rc);
    puts(buf);    
}

// *****************************************************************************
// cursor control
// 0 turns cursot off
// 1 
// *****************************************************************************
void LCD::lcd_cursor(char cv)
{
    switch(cv) {
        case 0: { // turn off cursor
            putch(_adr);
            puts("c12\r");
            break;
        }
        case 1: { // cursor on normal
            putch(_adr);
            puts("c14\r");
            break;
        }
        case 2: { // blok flashing
            putch(_adr);
            puts("c13\r");
            break;
        }
    }
}

// *****************************************************************************
// Print overloaded 
// *****************************************************************************
// String
void LCD::lcd_print(char *value)
{
    putch(_adr);
    putch('d');    
    puts(value);
    putch(13);    
    delay(1);
}
// Int
void LCD::lcd_print(int value)
{
char buf[15];
    sprintf(buf, "d%d", value);
    putch(_adr);
    puts(buf);
    putch(13);
    delay(1);    
}

// *****************************************************************************
// back light control
// RGB has 3 back lights
// *****************************************************************************
void LCD::lcd_bl(char red, char green, char blue)
{
char buf[15];
    if(red > 10) red = 10;
    if(green > 10 ) green = 10;
    if(blue > 10) blue =10;
    putch(_adr);
    sprintf(buf, "b%d,%d,%d\r",red,green,blue);
    puts(buf);
}

// *****************************************************************************
// define custom character
// *****************************************************************************
void LCD::lcd_defcust(char position, char *definition)
{
char adr = 0x40;
char buf[15], *bp=definition;
    if(position > 7) position = 7;
    adr += 8 * position;
    // set address using command c
    putch(_adr);
    sprintf(buf, "c%d\r", adr);
    puts(buf);
    delay(100);
    // now write definition
    putch(_adr);
    putch('d');
    putch(*(bp++));
    putch(*(bp++));
    putch(*(bp++));
    putch(*(bp++));
    putch(*(bp++));
    putch(*(bp++));
    putch(*(bp++));
    putch(*(bp++));
    putch(13);
    delay(100);
}

// *****************************************************************************
// prints a custom char
// *****************************************************************************
void LCD::lcd_printcust(char address)
{
char buf[15];
    if(address > 7) address = 7;
    putch(_adr);
    putch('d');
    putch(address);
    putch(13);
}    

